disp(' ')
disp(' fourier.m   ver 2.5   November 22, 2010')
disp(' by Tom Irvine   Email: tomirvine@aol.com')
disp(' ')
disp(' This program calculates the Fourier transform of a time history')
disp(' which is pre-loaded into Matlab.  ')
disp(' ')
disp(' The time history should have less than 10000 points')
disp(' ')
disp(' The time history must have two columns: time(sec) & amplitude(units)')
disp(' ')
%
clear t;
clear y;
clear z;
clear tmx;
clear tmi;
clear n;
clear dt;
clear df;
clear sr;
clear f_real;
clear f_imag;
clear freq;
clear ff;
clear zz;
clear mu;
clear hw;
clear ms;
clear rms;
clear THM;
clear PSD;
%
%  need mean removal and hanning
%
disp(' ')
disp(' Select file input method ');
disp('   1=external ASCII file ');
disp('   2=file preloaded into Matlab ');
disp('   3=Excel file ');
file_choice = input('');
%
if(file_choice==1)
        [filename, pathname] = uigetfile('*.*');
        filename = fullfile(pathname, filename);
        fid = fopen(filename,'r');
        THM = fscanf(fid,'%g %g',[2 inf]);
        THM=THM';
end
if(file_choice==2)
        THM = input(' Enter the matrix name:  ');
end
if(file_choice==3)
        [filename, pathname] = uigetfile('*.*');
        xfile = fullfile(pathname, filename);
%        
        THM = xlsread(xfile);
%         
end
%
t=double(THM(:,1));
y=double(THM(:,2));
%
tmx=max(t);
tmi=min(t);
mu=mean(y);
%
n = length(y)
%
if(n>10000)
    out1=sprintf('\n Warning: long calculation time required for high sample number.\n');
    disp(out1)
end
%
dt=(tmx-tmi)/(n-1);
sr=1./dt;
df=1./(tmx-tmi);
disp(' ')
disp(' Time Step ');
dtmin=min(diff(t));
dtmax=max(diff(t));
%
out4 = sprintf(' dtmin  = %8.4g sec  ',dtmin);
out5 = sprintf(' dt     = %8.4g sec  ',dt);
out6 = sprintf(' dtmax  = %8.4g sec  ',dtmax);
disp(out4)
disp(out5)
disp(out6)
%
disp(' ')
disp(' Sample Rate ');
out4 = sprintf(' srmin  = %8.4g samples/sec  ',1/dtmax);
out5 = sprintf(' sr     = %8.4g samples/sec  ',1/dt);
out6 = sprintf(' srmax  = %8.4g samples/sec  \n',1/dtmin);
disp(out4)
disp(out5)
disp(out6)
%
ncontinue=1;
%
if(((dtmax-dtmin)/dt)>0.01)
    disp(' ')
    disp(' Warning:  time step is not constant.  Continue calculation? 1=yes 2=no ')
    ncontinue=input(' ')
end
if(ncontinue==1)
%
%  Output options
%
disp(' ')
disp(' Select output option ');
choice=input(' 1=plot only   2=plot & output text file  ' );
disp(' ')
%
if choice==2
   disp(' Enter the complex output filename ');
   filename1 = input(' ','s');
%   
   disp(' ')  
   disp(' Enter the one-sided, full amplitude filename ');
   filename2 = input(' ','s'); 
%
   fid1 = fopen(filename1,'w');
   fid2 = fopen(filename2,'w');   
%   
end
%
disp(' ')
disp(' mean removal? ');
mr_choice=input(' 1=yes   2=no  ' );
disp(' ')
%
if(mr_choice==1)
    y=y-mu;
end
disp(' ')
disp(' Select Window ')
hw_choice = input(' 1=Rectangular 2=Hanning  ');
disp(' ')
%
   ae=((8/3)^(1/2));
%    
if(hw_choice==2)
    progressbar % Create figure and set starting time 
    for i=0:(n-1)
        progressbar(i/(n-1)) % Update figure  
        hw(i+1) =sin( (i*pi/n) )^2 ;
    end
     y=(y.*hw')*ae;
end
%
out1 = sprintf('\n  %d samples \n',n);
disp(out1)
%
out1 = sprintf(' SR  = %g samples/sec    dt = %g sec \n',sr,dt);
disp(out1)
%
iflag =0;
tpi=2.*pi;
%%%
ms=0;
jk=0;
kk=1;
%
disp(' ');
progressbar % Create figure and set starting time
%
for  k=0:(n-1)
  progressbar(k/n) % Update figure    
%
  clear sum_c;
  clear sum_s;
  clear ccc;
  clear sss;
%
  f_real(k+1)=0.;
  f_imag(k+1)=0.;  
%   
  arg = linspace(0,tpi*k*(n-1)/n,n); 
%
  ccc=y.*cos(arg)';
  sss=y.*sin(arg)';
%
  sum_c=sum(ccc);
  sum_s=sum(sss);
%
  f_real(k+1)=f_real(k+1)+sum_c;
  f_imag(k+1)=f_imag(k+1)+sum_s;
%
  freq(k+1)=k*df;
%       
  f_real(k+1)=f_real(k+1)/n;
  f_imag(k+1)=f_imag(k+1)/(-n);
%
  z(k+1)=((f_real(k+1)^2) + (f_imag(k+1)^2))^(1/2);
%  
  if choice==2
        fprintf(fid1,'%11.4e \t %11.4e \t %11.4e \n',freq(k+1), f_real(k+1), f_imag(k+1) );      
  end    
%		  
		  if(k<(n/2))  
%              
				 if(k > 0)			 
                    z(k+1)=2.*z(k+1);  

                 end
                 if choice==2
                    fprintf(fid2,'%11.4e \t %11.4e \n',freq(k+1), z(k+1) );
                 end
                 zz(kk)= z(k+1);
                 ff(kk)=freq(k+1);
                 ms=ms+(zz(kk))^2;     
                 kk=kk+1;
 %                
          end
  end   
  progressbar(1);
%
  if choice==2
       fclose(fid1);
       fclose(fid2);     
  end
%
disp(' ');
disp(' Plot Fourier Transform ');
%
    figure(1); 
    plot(ff,zz);
    grid on;
    xlabel('Frequency (Hz)');
    ylabel('Amplitude');    
    out5 = sprintf(' Fourier Transform Magnitude ');
    title(out5);  
    zoom on
%
    clear FT;
    FT=[ff',zz'];
%%%%
    rms=ms^(1/2);
%
    out1 = sprintf('\n  Fourier transform overall level = %g \n',rms);
    disp(out1)
%
    fmax=0;
    zmax=0;
    for(i=1:length(ff))
        if(zz(i)>zmax)
            zmax=zz(i);
            fmax=ff(i);
        end
    end
%
    out5 = sprintf('\n Peak occurs at %10.5g Hz ',fmax);
    disp(out5)
%
    out1=sprintf('\n Calculation complete. \n');
    disp(out1)
%%%%
end
%
clear complex_FT;
complex_FT=[freq', f_real', f_imag'];
%
disp(' ');
disp(' Plot PSD? ')
disp(' 1=yes 2=no');
ipp=input(' ');
%
if(ipp==1)
    clear PS;
    zz=zz/sqrt(2);
    PS=zz.*zz/df;
    ms=0.;
    for(i=1:length(PS))
        ms=ms+PS(i);
    end
    rms=sqrt(ms*df);
    sz=size(PS);
    if(sz(2)>sz(1))
        PS=PS';
    end
    PSD=[ff',PS];
    figure(2);
    plot(ff,PS);
    xlabel(' Frequency (Hz)');
    ylabel(' PSD (amplitude^2/Hz)'); 
    at = sprintf(' Power Spectral Density  Overall Level = %6.3g RMS ',rms);   
    title(at);
    set(gca,'MinorGridLineStyle','none','GridLineStyle',':','XScale','log','YScale','log'); 
    grid on;
%
    psd_max=max(PS);
    psd_min=min(PS);
%    
    if(psd_max < 10000)
        ymax=10000;
    end
    if(psd_max < 1000)
        ymax=1000;
    end
    if(psd_max < 100)
        ymax=100;
    end
    if(psd_max < 10)
        ymax=10;
    end
    if(psd_max < 1)
        ymax=1;
    end
    if(psd_max < 0.1)
        ymax=0.1;
    end
    if(psd_max < 0.01)
        ymax=0.01;
    end
    if(psd_max < 0.001)
        ymax=0.001;
    end
    if(psd_max < 0.0001)
        ymax=0.0001;
    end
    if(psd_max < 0.00001)
        ymax=0.00001;
    end
%
    ymin=ymax/100;    
    if(psd_min < psd_max/100000)
       ymin=ymax/100000;    
    end    
    fmax=max(ff);
    set(gca, 'XTickMode', 'manual');
%
    pfmax=10000;
    set(gca,'xtick',[10 100 1000 10000]); 
%
    pfmax=fmax;
    if(fmax<10000)
        pfmax=10000;
        set(gca,'xtick',[10 100 1000 10000]); 
    end
    if(fmax<5000)
        pfmax=5000;
        set(gca,'xtick',[10 100 1000 5000]);   
    end
    if(fmax<2000)
        pfmax=2000;
        set(gca,'xtick',[10 100 1000 2000]);      
    end
    if(fmax<1000)
        pfmax=1000;
        if(fmin<1)
           fmin=1; 
           set(gca,'xtick',[1 10 100 1000]);    
        else    
           set(gca,'xtick',[10 100 1000]);
        end   
    end
    fmin=df;
    axis([fmin,pfmax,ymin,ymax]);
    power_spectral_density=PSD;
    clear PSD;
%    
end
%
   disp('   complex Fourier transform saved as: complex_FT ');
   disp(' Fourier transform magnitude saved as: FT ');
if(ipp==1)
   disp('      Power spectral density saved as: power_spectral_density ');
end